import unittest

try:
    import vimba
    from vimba import Vimba, VimbaFeatureError
except Exception:
    raise unittest.SkipTest(
        "vimba not installed. Skipping all tests in avt_camera_test.py")

from pysilico_server.devices.avtCamera import AvtCamera
from time import sleep
import functools


def withCamera():

    def wrapperFunc(f):

        @functools.wraps(f)
        def wrapper(self, *args, **kwds):
            with Vimba.get_instance():
                with self._vimbacamera:
                    return f(self, *args, **kwds)

        return wrapper

    return wrapperFunc


class HwAvtCameraTest(unittest.TestCase):

    def setUp(self):
        with Vimba.get_instance() as v:
            self._vimbacamera = v.get_all_cameras()[0]
        self._cam = AvtCamera(self._vimbacamera, 'camera_name')

    def test_exposure_time(self):
        self._cam.setExposureTime(100)
        exp_time = self._cam.exposureTime()
        self.assertAlmostEqual(100, exp_time)
        self._cam.setExposureTime(200)
        exp_time = self._cam.exposureTime()
        self.assertAlmostEqual(200, exp_time)

    def test_after_initialize(self):
        self.assertAlmostEqual(10000000,
                               self._cam.getStreamBytesPerSecond())
        self.assertAlmostEqual(1,
                               self._cam.getBinning())

    @withCamera()
    def test_framerate(self):
        wanted = 2
        self._cam.startAcquisition()
        sleep(3)
        self._cam.setFrameRate(wanted)
        got = self._cam.getFrameRate()
        self._cam.stopAcquisition()
        self.assertAlmostEqual(wanted, got)

    def test_print_info(self):
        print('model name %s' % self._cam.deviceModelName())
        print('id %s' % self._cam.deviceID())
        print('ip address %s' % self._cam.ipAddress())


class VimbaTest(unittest.TestCase):

    def test_list_features(self):
        with Vimba.get_instance() as v:
            vimbacamera = v.get_all_cameras()[0]
            with vimbacamera:
                for feature in vimbacamera.get_all_features():
                    self._print_feature(feature)

    def _print_feature(self, feature):
        try:
            value = feature.get()

        except (AttributeError, VimbaFeatureError):
            value = None

        print('/// Feature name   : {}'.format(feature.get_name()))
        print('/// Display name   : {}'.format(feature.get_display_name()))
        print('/// Tooltip        : {}'.format(feature.get_tooltip()))
        print('/// Description    : {}'.format(feature.get_description()))
        print('/// SFNC Namespace : {}'.format(feature.get_sfnc_namespace()))
        print('/// Unit           : {}'.format(feature.get_unit()))
        print('/// Value          : {}\n'.format(str(value)))


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



import threading
import time
import numpy as np
from plico.utils.hackerable import Hackerable
from plico.utils.snapshotable import Snapshotable
from plico.utils.stepable import Stepable
from plico.utils.serverinfoable import ServerInfoable
from plico.utils.logger import Logger
from plico.utils.decorator import override, logEnterAndExit, synchronized
from plico.utils.timekeeper import TimeKeeper
from pysilico.types.camera_frame import CameraFrame
from pysilico.types.camera_status import CameraStatus
from rebin import rebin


class CameraController(Stepable,
                       Snapshotable,
                       Hackerable,
                       ServerInfoable):

    def __init__(self,
                 servername,
                 ports,
                 camera,
                 replySocket,
                 publisherSocket,
                 statusSocket,
                 displaySocket,
                 rpcHandler,
                 timeMod=time):
        self._camera = camera
        self._replySocket = replySocket
        self._publisherSocket = publisherSocket
        self._statusSocket = statusSocket
        self._displaySocket = displaySocket
        self._rpcHandler = rpcHandler
        self._timeMod = timeMod
        self._logger = Logger.of('CameraController')
        Hackerable.__init__(self, self._logger)
        ServerInfoable.__init__(self, servername,
                                ports,
                                self._logger)
        self._isTerminated = False
        self._stepCounter = 0
        self._frameCounter = 0
        self._lastDisplayTimestamp = 0
        self._displayIntervalInSec = 0.05
        self._timekeep = TimeKeeper()
        self._cameraStatus = None
        self._mutexStatus = threading.RLock()
        self._camera.registerCallback(self._publishFrame)
        self._darkFrame = None
        self._mutexDarkFrame = threading.RLock()

    @override
    def step(self):
        self._rpcHandler.handleRequest(self, self._replySocket, multi=True)
        self._publishStatus()
        if self._timekeep.inc():
            self._logger.notice(
                'Stepping at %5.2f Hz. FrameCounter %d' % (
                    self._timekeep.rate, self._camera.getFrameCounter()))
        self._stepCounter += 1

    def getStepCounter(self):
        return self._stepCounter

    def terminate(self):
        self._logger.notice("Got request to terminate")
        try:
            self._camera.stopAcquisition()
            self._camera.deinitialize()
        except Exception as e:
            self._logger.warn("Could not stop camera acquisition: %s" %
                              str(e))
        self._isTerminated = True

    @override
    def isTerminated(self):
        return self._isTerminated

    def getShape(self):
        assert False, 'Should not be used, client uses getStatus instead'

    def getDtype(self):
        assert False, 'Should not be used, client uses getStatus instead'

    @logEnterAndExit('Entering setExposureTime',
                     'Executed setExposureTime')
    def setExposureTime(self, exposureTimeInMilliSeconds):
        self._camera.setExposureTime(exposureTimeInMilliSeconds)
        with self._mutexStatus:
            self._cameraStatus = None

    def _getExposureTime(self):
        return self._getCameraStatus().exposureTimeInMilliSec

    @logEnterAndExit('Entering setBinning',
                     'Executed setBinning')
    def setBinning(self, binning):
        self._camera.setBinning(binning)
        with self._mutexStatus:
            self._cameraStatus = None

    def getBinning(self):
        assert False, 'Should not be used, client uses getStatus instead'

    def getDarkFrame(self):
        with self._mutexDarkFrame:
            return self._darkFrame

    @logEnterAndExit('Entering setDarkFrame',
                     'Executed setDarkFrame')
    def setDarkFrame(self, darkFrame):
        with self._mutexDarkFrame:
            self._darkFrame = darkFrame

    def _getCorrectedFrame(self, frame):
        if self._darkFrame is not None:
            with self._mutexDarkFrame:
                return CameraFrame.fromNumpyArray(
                    frame.toNumpyArray() - self._darkFrame.toNumpyArray(),
                    frame.counter())
        else:
            return frame

    def getSnapshot(self, prefix):
        assert False, 'Should not be used, client uses getStatus instead'

    def _publishFrame(self, frame):
        correctedFrame = self._getCorrectedFrame(frame)
        self._rpcHandler.sendCameraFrame(self._publisherSocket, correctedFrame)
        self._logger.debug('frame %d published' % correctedFrame.counter())
        self._publishForDisplay(correctedFrame)

    def _downsizeForDisplay(self, frame):
        DISPLAY_FRAME_SIZE = 256.
        minSize = np.min(frame.toNumpyArray().shape)
        downsizeBy = np.int(np.ceil(
            minSize / DISPLAY_FRAME_SIZE))
        if downsizeBy > 1:
            array = self._downsizeBySampling(frame.toNumpyArray(),
                                             downsizeBy)
        else:
            array = frame.toNumpyArray()
        return CameraFrame.fromNumpyArray(np.float32(array),
                                          frame.counter())

    def _downsizeByRebin(self, frame, factor):
        return rebin(frame, factor) * factor**2

    def _downsizeBySampling(self, frame, factor):
        return frame[::factor, ::factor]

    def _publishForDisplay(self, frame):
        now = self._timeMod.time()
        if now - self._lastDisplayTimestamp < self._displayIntervalInSec:
            return
        downsizedFrame = self._downsizeForDisplay(frame)
        self._rpcHandler.sendCameraFrame(self._displaySocket,
                                         downsizedFrame)
        self._lastDisplayTimestamp = now

    @synchronized("_mutexStatus")
    def _getCameraStatus(self):
        if self._cameraStatus is None:
            self._logger.debug('get CameraStatus')
            self._cameraStatus = CameraStatus(
                self._camera.name(),
                self._camera.cols(),
                self._camera.rows(),
                self._camera.dtype(),
                self._camera.getBinning(),
                self._camera.exposureTime(),
                self._camera.getFrameRate())
        return self._cameraStatus

    def _publishStatus(self):
        self._rpcHandler.publishPickable(self._statusSocket,
                                         self._getCameraStatus())

    @logEnterAndExit('Entering setFrameRate',
                     'Executed setFrameRate')
    def setFrameRate(self, frameRate):
        self._camera.setFrameRate(frameRate)
        with self._mutexStatus:
            self._cameraStatus = None



#!/usr/bin/env python

import numpy as np
import textwrap
import threading
from plico.utils.decorator import logEnterAndExit, \
    synchronized, override
from pysilico_server.devices.abstract_camera import AbstractCamera
from plico.utils.logger import Logger
from pysilico.types.camera_frame import CameraFrame
from vimba import Vimba
import functools
from vimba.frame import PixelFormat, FrameStatus
from vimba.error import VimbaFeatureError


def withVimba():

    def wrapperFunc(f):

        @functools.wraps(f)
        def wrapper(self, *args, **kwds):
            with Vimba.get_instance():
                return f(self, *args, **kwds)

        return wrapper

    return wrapperFunc


def withCamera():

    def wrapperFunc(f):

        @functools.wraps(f)
        @withVimba()
        def wrapper(self, *args, **kwds):
            with self._camera:
                return f(self, *args, **kwds)

        return wrapper

    return wrapperFunc


class AvtCamera(AbstractCamera):

    VIMBA_BINNING_HORIZONTAL = "BinningHorizontal"
    VIMBA_BINNING_VERTICAL = "BinningVertical"
    VIMBA_DECIMATION_HORIZONTAL = 'DecimationHorizontal'
    VIMBA_DECIMATION_VERTICAL = 'DecimationVertical'
    VIMBA_FRAME_STATUS_COMPLETE = 0

    def __init__(self, vimbacamera, name):
        self._name = name
        self._camera = vimbacamera
        self._logger = Logger.of('AvtCamera')
        self._binning = 1
        self._counter = 0
        self._isContinuouslyAcquiring = False
        self._callbackList = []
        self._mutex = threading.RLock()
        self._lastValidFrame = CameraFrame(np.zeros((4, 4)), counter=0)
        self._initialize()

    @withVimba()
    def _initialize(self):

        # Limit data rate from camera
        try:
            streamBytesPerSecond = conf().getint('camera', 'streamBytesPerSecond')
        except:
            streamBytesPerSecond = 10000000
        self.setStreamBytesPerSecond(streamBytesPerSecond)

        self._resetBinningAndOffset()

        if self.pixelFormat() == PixelFormat.Mono12:
            self.BYTES_PER_PIXEL = 2
            self._dtype = np.uint16
        else:
            raise Exception('Format %s is not supported' % self.pixelFormat())

        self._logCameraInfo()
        self._logger.notice('AVT camera initialized')

    @logEnterAndExit('Entering _createFrames',
                     'Executed _createFrames',
                     'debug')
    @synchronized("_mutex")
    @withCamera()
    def _createFrames(self):
        # create new frames for the camera
        self._frame = self._camera.getFrame()  # creates a frame
        self._frame.announceFrame()

    def _isBinningAvailable(self):
        feat = list(self.getCameraFeatures().keys())
        return self.VIMBA_BINNING_HORIZONTAL in feat and \
            self.VIMBA_BINNING_VERTICAL in feat

    def _isDecimationAvailable(self):
        feat = list(self.getCameraFeatures().keys())
        return self.VIMBA_DECIMATION_HORIZONTAL in feat and \
            self.VIMBA_DECIMATION_VERTICAL in feat

    @synchronized("_mutex")
    @withCamera()
    def _resetBinningAndOffset(self):

        restartAcquistion = False
        if self._isContinuouslyAcquiring:
            self.stopAcquisition()
            restartAcquistion = True

        if self._isBinningAvailable():
            self._camera.BinningHorizontal.set(self._binning)
            self._camera.BinningVertical.set(self._binning)
        elif self._isDecimationAvailable():
            self._camera.DecimationHorizontal.set(self._binning)
            self._camera.DecimationVertical.set(self._binning)
        else:
            raise Exception("Neither binning nor decimation available")

        self._camera.OffsetX.set(0)
        self._camera.OffsetY.set(0)
        self._camera.Height.set(
            self._camera.SensorHeight.get() // self._binning)
        self._camera.Width.set(self._camera.SensorWidth.get() // self._binning)
        self._camera.set_pixel_format(PixelFormat.Mono12)
        self._camera.GVSPPacketSize.set(1500)
        # Not all cameras have this
        #self._timeStampTickFrequency = self._camera.GevTimestampTickFrequency.get()
        self._logger.notice(
            'Binning set to %d. Frame shape (w,h): (%d, %d) '
            'Left bottom pixel (%d, %d)'
            % (self._binning, self._camera.Width.get(),
               self._camera.Height.get(),
               self._camera.OffsetX.get(), self._camera.OffsetY.get()))

        if restartAcquistion:
            self.startAcquisition()

    @synchronized("_mutex")
    @withCamera()
    def _logCameraInfo(self):
        self._logger.notice('Camera: %s at %s - ID: %s' % (
                            self.deviceModelName(),
                            self.ipAddress(),
                            self.deviceID()))
        self._logger.notice('Sensor is %d rows x %d cols, %d bits/pixel' % (
            self._camera.SensorHeight.get(),
            self._camera.SensorWidth.get(), self._camera.SensorBits.get()))
        self._logger.notice('Output format is %s' % self.pixelFormat())
        self._logger.notice('Exposure time is %f ms' % self.exposureTime())

    @synchronized("_mutex")
    @withCamera()
    def setStreamBytesPerSecond(self, streamBytesPerSecond):
        try:
            self._camera.StreamBytesPerSecond.set(streamBytesPerSecond)
        except AttributeError:
            try:
                # Some cameras use a different name
                self._camera.DeviceLinkThroughputLimit.set(streamBytesPerSecond)
            except AttributeError:
                # If we can't set it, return silently and
                # avoid the logging notice
                return
        self._logger.notice('Camera data rate set to %4.1f MB/s'
                             % (streamBytesPerSecond / 1e6))

    @synchronized("_mutex")
    @withCamera()
    def getStreamBytesPerSecond(self):
        try:
            return self._camera.StreamBytesPerSecond.get()
        except AttributeError:
            # Some cameras do not have this attribute
            return 0

    @override
    def setBinning(self, binning):
        self._binning = binning
        self._resetBinningAndOffset()

    @override
    def getBinning(self):
        return self._binning

    @synchronized("_mutex")
    @withCamera()
    def deviceModelName(self):
        # return self._camera.DeviceModelName.get()
        return self._camera.get_model()

    @synchronized("_mutex")
    @withCamera()
    def deviceID(self):
        # return self._camera.DeviceID.get()
        return self._camera.get_id()

    @override
    def name(self):
        return self._name

    @override
    @synchronized("_mutex")
    @withCamera()
    def rows(self):
        return self._camera.Height.get()

    @override
    @synchronized("_mutex")
    @withCamera()
    def cols(self):
        return self._camera.Width.get()

    @synchronized("_mutex")
    @withCamera()
    def bpp(self):
        return self._camera.SensorBits.get()

    @synchronized("_mutex")
    @withCamera()
    def pixelFormat(self):
        return self._camera.get_pixel_format()

    @override
    def dtype(self):
        return self._dtype

    @override
    @synchronized("_mutex")
    @withCamera()
    def setExposureTime(self, exposureTimeInMilliSeconds):
        try:
            self._camera.ExposureTimeAbs.set(exposureTimeInMilliSeconds * 1000.)
        except AttributeError:
            # Some cameras use ExposureTime, others ExposureTimeAbs. We try both.
            self._camera.ExposureTime.set(exposureTimeInMilliSeconds * 1000.)
        self._logger.notice('Exposure time set to %g ms' % (
            exposureTimeInMilliSeconds))

    @override
    @synchronized("_mutex")
    @withCamera()
    def exposureTime(self):
        try:
            return self._camera.ExposureTimeAbs.get() / 1000.
        except AttributeError:
            # Some cameras use ExposureTime, others ExposureTimeAbs. We try both.
            return self._camera.ExposureTime.get() / 1000.

    @override
    @synchronized("_mutex")
    @withCamera()
    def getFrameRate(self):
        try:
            return self._camera.AcquisitionFrameRateAbs.get()
        except AttributeError:
            # Some cameras use AcquisitionFrameRate, others AcquisitionFrameRateAbs. We try both.
            return self._camera.AcquisitionFrameRate.get()

    @override
    @synchronized("_mutex")
    @withCamera()
    def setFrameRate(self, frameRate):
        try:
            self._camera.AcquisitionFrameRateAbs.set(np.minimum(
                frameRate,
                self._maximum_frame_rate()))
        except AttributeError:
            # Some cameras use AcquisitionFrameRate, others AcquisitionFrameRateAbs. We try both.
            self._camera.AcquisitionFrameRate.set(np.minimum(
                frameRate,
                self._maximum_frame_rate()))
        self._logger.notice('Frame rate set to %g Hz - (requested %g Hz)'
                            % (self.getFrameRate(), frameRate))

    def readFrame(self, timeoutMilliSec=2000):
        pass

    def _notifyListenersAboutNewFrame(self):
        for callback in self._callbackList:
            callback(self._lastValidFrame)

    def _frame_callback(self, camera, frame):
        try:
            # self._logger.debug("Got frame %d at time %.3f" % (
            #    self._counter, frame.get_timestamp() /
            #    self._timeStampTickFrequency))
            if frame.get_status() == FrameStatus.Complete:
                frame_data = frame.get_buffer()
                img = np.ndarray(buffer=frame_data,
                                 dtype=self._dtype,
                                 shape=(frame.get_height(), frame.get_width()))
                self._lastValidFrame = CameraFrame(img, counter=self._counter)
                self._notifyListenersAboutNewFrame()
                self._counter += 1
            else:
                self._logger.warn(
                    "Frame status not complete. "
                    "Try to reduce streamBytesPerSecond value")
            camera.queue_frame(frame)
        except Exception as e:
            self._logger.warn("Exception in handling frame callback: %s" %
                              str(e))

    @withCamera()
    def _adjust_packet_size(self):
        try:
            self._camera.GVSPAdjustPacketSize.run()

            while not self._camera.GVSPAdjustPacketSize.is_done():
                pass

        except (AttributeError, VimbaFeatureError):
            pass

    def _maximum_frame_rate(self):
        aLittleBitSlower = 0.01
        return self._camera.AcquisitionFrameRateLimit.get() - aLittleBitSlower

    @withCamera()
    def startAcquisition(self):
        self._adjust_packet_size()
        self._camera.TriggerSelector.set('FrameStart')
        self._camera.TriggerSource.set('FixedRate')
        self._camera.AcquisitionMode.set('Continuous')
        self.setFrameRate(self._maximum_frame_rate())
        try:
            self._camera.SyncOutSelector.set('SyncOut1')
            self._camera.SyncOutSource.set('Exposing')
        except AttributeError:
            pass
        self._camera.start_streaming(
            handler=self._frame_callback, buffer_count=10)
        self._logger.notice('Continuous acquisition started')
        self._isContinuouslyAcquiring = True

    @withCamera()
    def stopAcquisition(self):
        self._isContinuouslyAcquiring = False
        # self._stopAcquisitionAndFlushQueue()
        # time.sleep(0.2)
        # self._revokeFrames()
        self._camera.stop_streaming()

    @withCamera()
    def _stopAcquisitionAndFlushQueue(self):
        self._camera.runFeatureCommand('AcquisitionStop')
        self._camera.flushCaptureQueue()
        self._camera.endCapture()

    @withCamera()
    def _revokeFrames(self):
        self._camera.revokeAllFrames()

    @synchronized("_mutex")
    @withCamera()
    def ipAddress(self):
        ip = self._camera.GevCurrentIPAddress.get()
        return '.'.join([str(int('0x' + x, 16)) for x in reversed(
            textwrap.wrap(hex(ip), 2)[1:])])

    @synchronized("_mutex")
    @withCamera()
    def getCameraFeatures(self):
        res = {}
        for feature in self._camera.get_all_features():
            try:
                featureName = feature.get_name()
                featureValue = feature.get()
            except Exception:
                featureValue = None
            res[featureName] = featureValue
        return res

    @override
    def registerCallback(self, callback):
        self._callbackList.append(callback)

    @override
    def getFrameCounter(self):
        return self._counter

    @override
    @withCamera()
    def deinitialize(self):
        try:
            self.stopAcquisition()
            # There is no closeCamera() method apparently
            # self._camera.closeCamera()
        except Exception as e:
            self._logger.warn('Failed to close camera:'+str(e))



import unittest

try:
    import vimba
except Exception:
    raise unittest.SkipTest(
        "vimba not installed. Skipping all tests in avt_camera_test.py")

import numpy as np
from pysilico_server.devices.avtCamera import Vimba, AvtCamera
from vimba.frame import PixelFormat


class MyVimbaStructureFrame():

    def __init__(self):
        self.receiveStatus = 0


class MyVimbaFrame(object):

    def __init__(self, h, w, dtype):
        self._h = h
        self._w = w
        self._dtype = dtype
        self._frame = MyVimbaStructureFrame()

    def announceFrame(self):
        pass

    def queueFrameCapture(self):
        self._frame.receiveStatus = 1

    def waitFrameCapture(self, timeout=2000):
        self._frame.receiveStatus = 0
        return 0

    def getBufferByteData(self):
        return memoryview(np.ones((self._h, self._w), dtype=self._dtype))

    @property
    def height(self):
        return self._h

    @property
    def width(self):
        return self._w


class MyFeature():

    def __init__(self, name, value):
        self._name = name
        self._value = value

    def get_name(self):
        return self._name

    def get(self):
        return self._value

    def set(self, value):
        self._value = value


class MyVimbaCamera(object):
    IP_ADDRESS = '193.206.155.159'
    SENSOR_SIZE_H = 1024
    SENSOR_SIZE_W = 768

    def __init__(self):

        self.set_pixel_format(PixelFormat.Mono12)
        self._deviceModelName = 'bar'
        self._deviceID = '02-2060C-06184'
        self._feat_dict = {}
        feats = (
            MyFeature('GevCurrentIPAddress', int(2677788353)),
            MyFeature('GevTimestampTickFrequency', int(10000)),
            MyFeature('SensorHeight', self.SENSOR_SIZE_H),
            MyFeature('SensorWidth', self.SENSOR_SIZE_W),
            MyFeature('SensorBits', 12),
            MyFeature('ExposureTimeAbs', 10000.),
            MyFeature('StreamBytesPerSecond', 1000000),
            MyFeature('OffsetX', 0),
            MyFeature('OffsetY', 0),
            MyFeature('Height', 100),
            MyFeature('Width', 200),
            MyFeature('GVSPPacketSize', 1500),
        )
        for f in feats:
            self._feat_dict[f.get_name()] = f

        self.enableBinning()
        self.disableDecimation()

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass

    def __getattr__(self, attr):
        if attr[0] != '_':
            try:
                return self._feat_dict[attr]
            except KeyError:
                raise AttributeError
        else:
            try:
                return self.__dict__[attr]
            except KeyError:
                raise AttributeError(attr)

    def __setattr__(self, name, value):
        if name[0] != '_':
            self._checkDecimationEnabled(name, value)
            self._checkBinningEnabled(name, value)
            self._feat_dict[name] = MyFeature(name, value)
        else:
            self.__dict__[name] = value

    def _checkDecimationEnabled(self, name, value):
        if name in [AvtCamera.VIMBA_DECIMATION_HORIZONTAL,
                    AvtCamera.VIMBA_DECIMATION_VERTICAL]:
            if self._decimationEnabled is False:
                raise KeyError(name)

    def _checkBinningEnabled(self, name, value):
        if name in [AvtCamera.VIMBA_BINNING_HORIZONTAL,
                    AvtCamera.VIMBA_BINNING_VERTICAL]:
            if self._binningEnabled is False:
                raise KeyError(name)

    def get_all_features(self):
        dicto = self._feat_dict.values()
        return dicto

    # def openCamera(self):
    #     pass
    #
    # def startCapture(self):
    #     pass
    #
    # def endCapture(self):
    #     pass
    #
    # def flushCaptureQueue(self):
    #     pass
    #
    # def runFeatureCommand(self, command):
    #     pass
    #
    # def getFrame(self):
    #     return MyVimbaFrame(self.SensorHeight, self.SensorWidth, np.uint16)
    #
    # def revokeAllFrames(self):
    #     pass

    def enableBinning(self):
        self._binningEnabled = True
        self.BinningHorizontal = 1
        self.BinningVertical = 1

    def disableBinning(self):
        self._binningEnabled = False
        self._feat_dict.pop(AvtCamera.VIMBA_BINNING_HORIZONTAL, None)
        self._feat_dict.pop(AvtCamera.VIMBA_BINNING_VERTICAL, None)

    def enableDecimation(self):
        self._decimationEnabled = True
        self.DecimationHorizontal = 1
        self.DecimationVertical = 1

    def disableDecimation(self):
        self._decimationEnabled = False
        self._feat_dict.pop(AvtCamera.VIMBA_DECIMATION_HORIZONTAL, None)
        self._feat_dict.pop(AvtCamera.VIMBA_DECIMATION_VERTICAL, None)

    def set_pixel_format(self, format):
        self._pixel_format = format

    def get_pixel_format(self):
        return self._pixel_format

    def get_id(self):
        return self._deviceID

    def get_model(self):
        return self._deviceModelName


class TestAvtCamera(unittest.TestCase):

    def setUp(self):
        self.vimbacamera = MyVimbaCamera()
        self.avt = AvtCamera(self.vimbacamera, 'foo')

    def test_creation(self):
        self.assertEqual(MyVimbaCamera.IP_ADDRESS, self.avt.ipAddress())

    def testSetBinningUsingDecimation(self):
        self.vimbacamera.disableBinning()
        self.vimbacamera.enableDecimation()
        self.avt.setBinning(1)
        self.assertEqual(1, self.vimbacamera.DecimationHorizontal.get())
        self.assertEqual(1, self.vimbacamera.DecimationVertical.get())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_H, self.avt.rows())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_W, self.avt.cols())
        self.avt.setBinning(4)
        self.assertEqual(4, self.vimbacamera.DecimationHorizontal.get())
        self.assertEqual(4, self.vimbacamera.DecimationVertical.get())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_H / 4, self.avt.rows())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_W / 4, self.avt.cols())
        self.assertEqual(4, self.avt.getBinning())

    def testSetBinningWithoutDecimation(self):
        self.vimbacamera.enableBinning()
        self.vimbacamera.disableDecimation()
        self.avt.setBinning(1)
        self.assertEqual(1, self.vimbacamera.BinningHorizontal.get())
        self.assertEqual(1, self.vimbacamera.BinningVertical.get())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_H, self.avt.rows())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_W, self.avt.cols())
        self.avt.setBinning(4)
        self.assertEqual(4, self.vimbacamera.BinningHorizontal.get())
        self.assertEqual(4, self.vimbacamera.BinningVertical.get())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_H / 4, self.avt.rows())
        self.assertEqual(self.vimbacamera.SENSOR_SIZE_W / 4, self.avt.cols())
        self.assertEqual(4, self.avt.getBinning())

    def testSetBinningWithoutDecimationNorBinningRaisesException(self):
        self.vimbacamera.disableBinning()
        self.vimbacamera.disableDecimation()
        self.assertRaises(Exception, self.avt.setBinning, 1)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import numpy as np
import time
from rebin import rebin
from pysilico_server.devices.base_simulated_camera import BaseSimulatedCamera

__version__ = "$Id: simulated_camera.py 203 2017-01-16 11:30:46Z lbusoni $"


class SimulatedAuxiliaryCamera(BaseSimulatedCamera):

    def __init__(self, name='SimulatedAuxiliaryCamera'):
        BaseSimulatedCamera.__init__(self, name)
        self._totalFluxPerMilliSecond = 1e5
        self.setFrameRate(3)

    def _makeLetterF(self, h, w):
        pixels = np.zeros((h, w), dtype=float)
        b = min(h, w) // 7

        pixels[b:6 * b, int(w / 2 - 1.5 * b):int(w / 2 - 0.5 * b)] = 1
        pixels[b:2 * b, int(w / 2 - 1.5 * b):int(w / 2 + 2.0 * b)] = 1
        pixels[3 * b:4 * b, int(w / 2 - 1.5 * b):int(w / 2 + 2.0 * b)] = 1
        return pixels

    def _makeCentralSpot(self, h, w):
        pixels = np.zeros((h, w), dtype=np.float)
        pixels[int(h / 2): int(h / 2 + 1),
               int(w / 2): int(w / 2 + 1)] = 1.0
        return pixels

    def _computeFrameFromWavefront(self):
        h, w = (self.rows(), self.cols())

        pixels = self._makeLetterF(h, w)

        normalize = self._totalFluxPerMilliSecond * \
            self.exposureTime() / pixels.sum()
        pixels *= normalize

        if self._noiseInCount > 0:
            pixels += np.random.normal(
                3 * self._noiseInCount,
                self._noiseInCount,
                size=(h, w))

        # if self.getBinning() != 1:
        #    pixels= rebin(pixels, self.getBinning()) * \
        #        self.getBinning()**2
        pixels = pixels.clip(min=0, max=self.MAX_VALUE)

        self._counter += 1
        time.sleep(1. / self.getFrameRate())
        return pixels.astype(self.DTYPE)



import numpy as np
import time
from plico.utils.decorator import override, returns
from pysilico_server.devices.abstract_camera import AbstractCamera
from plico.utils.logger import Logger
from plico.utils.concurrent_loop import ConcurrentLoop
from plico.utils.convergeable import Convergeable
from pysilico.types.camera_frame import CameraFrame


__version__= "$Id: $"



class SimulatedFrameProducer(Convergeable):

    def __init__(self, simulatedCamera):
        self._camera= simulatedCamera

        self._cycleCounter= 0
        self._logger= Logger.of("SimulatedFrameProducer")


    @override
    @returns(bool)
    def hasConverged(self):
        return False


    @override
    def performOneConvergenceStep(self):
        self._camera.produceFrame()
        self._cycleCounter += 1


    @override
    def measureConvergence(self):
        pass




class BaseSimulatedCamera(AbstractCamera):
    SENSOR_H= 1024
    SENSOR_W= 1360
    DTYPE= np.uint16
    MAX_VALUE= 4096


    def __init__(self, name='Simulated Camera'):
        self._name= name
        self._logger= Logger.of('SimulatedCamera')
        self._counter= 0
        self._delay= 0.00
        self._exposureTimeMs= 10.0
        self._frameRate= 1000./ self._exposureTimeMs
        self._binning= 1
        self._totalFluxPerMilliSecond= 2e6
        self._noiseInCount= 10.

        self._lastValidFrame= None
        self._callbackList= []
        self._buildFrameProducerLoop()

        self._logger.notice('Simulated camera initialized')
        self._logger.notice('Sensor size (H,W) %s %s' % (
            self.SENSOR_H, self.SENSOR_W))
        self._raiseExceptionOnDeinitialize= False


    def raiseExceptionOnDeinitialize(self, trueOrFalse):
        self._raiseExceptionOnDeinitialize= trueOrFalse


    def _buildFrameProducerLoop(self):
        self._frameProducer= SimulatedFrameProducer(self)
        self._frameProducerLoop= ConcurrentLoop(
            "SimulatedFrameProducerLoop", self._frameProducer, 0.01,
            self._logger.error)
        self._frameProducerLoop.initialize()


    def _notifyListenersAboutNewFrame(self):
        for callback in self._callbackList:
            callback(self._lastValidFrame)


    def produceFrame(self):
        frame= self._computeFrameFromWavefront()
        self._lastValidFrame= CameraFrame(frame, self._counter)
        self._notifyListenersAboutNewFrame()
        self._counter+= 1


    @override
    def deinitialize(self):
        if self._raiseExceptionOnDeinitialize:
            raise Exception('Asked to fail on deinitialize')
        else:
            self._frameProducerLoop.deinitialize()


    @override
    def startAcquisition(self):
        self._frameProducerLoop.close()


    @override
    def stopAcquisition(self):
        self._frameProducerLoop.open()


    @override
    def registerCallback(self, callback):
        self._callbackList.append(callback)


    def frameCounter(self):
        return self._counter


    @override
    def name(self):
        return self._name


    @override
    def rows(self):
        return int(float(self.SENSOR_H)/ self._binning)


    @override
    def cols(self):
        return int(float(self.SENSOR_W)/ self._binning)


    @override
    def dtype(self):
        return self.DTYPE


    @override
    @returns(np.ndarray)
    def readFrame(self, timeoutMilliSec=2000):
        return self._computeFrameFromWavefront()


    def setNoiseInCount(self, noise):
        self._noiseInCount= noise


    def setTotalFluxPerMilliSecond(self, totalFluxPerMilliSec):
        self._totalFluxPerMilliSecond= totalFluxPerMilliSec


    def _computeFrameFromWavefront(self):
        pixels = np.zeros((self.rows(), self.cols()),
                          dtype=np.float)

        self._counter += 1
        time.sleep(1./ self.getFrameRate())
        return pixels.astype(self.DTYPE)


    @override
    def setExposureTime(self, exposureTimeInMilliSeconds):
        self._exposureTimeMs= exposureTimeInMilliSeconds


    @override
    def exposureTime(self):
        return self._exposureTimeMs


    @override
    def setBinning(self, binning):
        self._logger.notice("setting binning to %d" % binning)
        self._binning= binning


    @override
    def getBinning(self):
        return self._binning


    @override
    def getFrameRate(self):
        return self._frameRate


    @override
    def setFrameRate(self, frameRate):
        self._frameRate= frameRate


    @override
    def getFrameCounter(self):
        return self._counter



#!/usr/bin/env python
import logging
from plico.utils.kill_process_by_name import killProcessByName
from pysilico_server.utils.constants import Constants

__version__= "$Id: tipico_kill_processes.py 31 2018-01-27 10:47:29Z lbusoni $"



def main():
    logging.basicConfig(level=logging.INFO)
    processNames= [Constants.START_PROCESS_NAME,
                   Constants.SERVER_PROCESS_NAME,
                   ]

    for each in processNames:
        killProcessByName(each)



import abc
from six import with_metaclass
import numpy
from plico.utils.decorator import returns


__version__= "$Id: abstract_camera.py 268 2017-04-12 21:54:14Z lbusoni $"


class AbstractCamera(with_metaclass(abc.ABCMeta, object)):

    @abc.abstractmethod
    def name(self):
        assert False

    @abc.abstractmethod
    @returns(numpy.ndarray)
    def readFrame(self, timeoutMilliSec=2000):
        assert False

    @abc.abstractmethod
    def rows(self):
        assert False

    @abc.abstractmethod
    def cols(self):
        assert False

    @abc.abstractmethod
    def dtype(self):
        assert False

    @abc.abstractmethod
    def setExposureTime(self, exposureTimeInMilliSeconds):
        assert False

    @abc.abstractmethod
    def exposureTime(self):
        assert False

    @abc.abstractmethod
    def setBinning(self, binning):
        assert False

    @abc.abstractmethod
    def getBinning(self):
        assert False

    @abc.abstractmethod
    def registerCallback(self):
        assert False

    @abc.abstractmethod
    def startAcquisition(self):
        assert False

    @abc.abstractmethod
    def stopAcquisition(self):
        assert False

    @abc.abstractmethod
    def getFrameCounter(self):
        assert False

    @abc.abstractmethod
    def deinitialize(self):
        assert False



import os

__version__= "$Id: process_startup_helper.py 26 2018-01-26 19:06:25Z lbusoni $"


class ProcessStartUpHelper(object):

    def __init__(self):
        self._moduleRoot= 'pysilico_server'


    def cameraControllerStartUpScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'camera_controller',
                            'pysilico_run_camera_controller.py')


    def killAllProcessesStartUpScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'utils',
                            'pysilico_kill_processes.py')


    def processProcessMonitorStartUpScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'process_monitor',
                            'pysilico_run_process_monitor.py')


    def processProcessMonitorStopScriptPath(self):
        return os.path.join(self._moduleRoot,
                            'utils',
                            'pysilico_server_stop.py')



#!/usr/bin/env python
import unittest
from pysilico_server.devices.simulated_auxiliary_camera import \
    SimulatedAuxiliaryCamera
from pysilico_server.camera_controller.camera_controller import \
    CameraController

__version__ = "$Id: camera_controller_test.py 293 2017-06-21 17:10:57Z lbusoni $"


class MyReplySocket():
    pass


class MyPublisherSocket():
    pass


class MyRpcHandler():

    def __init__(self):
        self._publish= {}


    def handleRequest(self, obj, socket, multi):
        pass


    def publishPickable(self, socket, anObject):
        self._publish[socket]= anObject


    def getLastPublished(self, socket):
        return self._publish[socket]


    def sendCameraFrame(self, socket, frame):
        self.publishPickable(socket, frame)


class CameraControllerTest(unittest.TestCase):

    def setUp(self):
        self._camera= SimulatedAuxiliaryCamera('qfs')
        self._rpcHandler= MyRpcHandler()
        self._replySocket= MyReplySocket()
        self._publisherSocket= MyPublisherSocket()
        self._statusSocket= MyPublisherSocket()
        self._displaySocket= MyPublisherSocket()
        self._serverName= 'pippo'
        self._ports= 'foo'
        self._ctrl= CameraController(
            self._serverName,
            self._ports,
            self._camera,
            self._replySocket,
            self._publisherSocket,
            self._statusSocket,
            self._displaySocket,
            self._rpcHandler)


    def tearDown(self):
        self._camera.raiseExceptionOnDeinitialize(False)
        self._camera.deinitialize()


    def testPublishesStatus(self):
        self._ctrl.step()
        status= self._rpcHandler.getLastPublished(
            self._statusSocket)
        self.assertEqual(self._camera.exposureTime(),
                         status.exposureTimeInMilliSec)
        self.assertEqual(self._camera.cols(),
                         status.frameWidth)
        self.assertEqual(self._camera.dtype(),
                         status.dtype)


    def testReadCameraStatusOnlyIfNeeded(self):
        self._ctrl.setBinning(1)
        self._ctrl.step()
        status= self._rpcHandler.getLastPublished(
            self._statusSocket)
        self._ctrl.step()
        status2= self._rpcHandler.getLastPublished(
            self._statusSocket)
        self.assertEqual(status, status2)
        self._ctrl.setBinning(2)
        self._ctrl.step()
        status3= self._rpcHandler.getLastPublished(
            self._statusSocket)
        self.assertNotEqual(status, status3)


    def testTerminate(self):
        self._camera.raiseExceptionOnDeinitialize(True)
        self._ctrl.terminate()
        self.assertTrue(self._ctrl.isTerminated())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


#!/usr/bin/env python

import numpy as np
import time
from rebin import rebin
from plico.utils.zernike_generator import ZernikeGenerator
from plico.types.zernike_coefficients import ZernikeCoefficients
from pysilico_server.devices.base_simulated_camera import BaseSimulatedCamera

__version__ = "$Id: simulated_camera.py 293 2017-06-21 17:10:57Z lbusoni $"


class SimulatedPyramidWfsCamera(BaseSimulatedCamera):
    SENSOR_H = 1024
    SENSOR_W = 1360
    DTYPE = np.uint16
    MAX_VALUE = 4096

    def __init__(self, name='Simulated Pyramid Wfs Camera'):
        BaseSimulatedCamera.__init__(self, name)
        self._pupilRadius = None
        pupilsSeparation = 520
        opticalAxis = (self.SENSOR_H / 2, self.SENSOR_W / 2)
        self._pupilsCenter = self._computeCenters(pupilsSeparation,
                                                 opticalAxis)
        self._tipTiltCoefficients = np.zeros(2)
        self._wavefront = None
        self._zernikeGenerator = None

        self.setPupilsRadiusInUnbinnedPixels(80)
        self.setScaleInMeterPerPixel(
            10e-3 / (2 * self.getPupilsRadiusInUnbinnedPixels()))
        self.setSlopeSaturationInRadians(40e-3)

        self._logger.notice('Pupil centers %s' % (self._pupilsCenter))
        self._logger.notice('Pupil radius %s' % (self._pupilRadius))

    def setPupilsRadiusInUnbinnedPixels(self, radius):
        self._pupilRadius = radius
        self._wavefront = np.zeros((2 * self._pupilRadius, 2 * self._pupilRadius))
        self._zernikeGenerator = ZernikeGenerator(2 * self._pupilRadius)

    def getPupilsRadiusInUnbinnedPixels(self):
        return self._pupilRadius

    def setTilt(self, tipTiltCoefficients):
        self._tipTiltCoefficients = tipTiltCoefficients

    def setWavefront(self, wavefront):
        self._wavefront = wavefront

    def _computeWavefrontFromZernikeCoefficients(self, coeff, zg):
        wf = 0.* zg.getZernike(1)
        for y in coeff.zernikeIndexes():
            wf += coeff.getZ([y]) * zg.getZernike(y)
        return wf

    def setWavefrontFromZernikeVector(self, zernikeVector):
        wf = self._computeWavefrontFromZernikeCoefficients(
            ZernikeCoefficients.fromNumpyArray(np.array(zernikeVector)),
            self._zernikeGenerator)
        self.setWavefront(wf)

    def _computeCenters(self, pupilsSeparation, opticalAxis):
        pupDist = int(0.5 * pupilsSeparation)
        center = opticalAxis
        res = np.zeros((4, 2))
        res[0,:] = [center[0] + pupDist, center[1] - pupDist]
        res[1,:] = [center[0] + pupDist, center[1] + pupDist]
        res[2,:] = [center[0] - pupDist, center[1] - pupDist]
        res[3,:] = [center[0] - pupDist, center[1] + pupDist]
        return res

    def setPupilsCenterInUnbinnedPixels(self, centers):
        assert centers.shape == (4, 2)
        self._pupilsCenter = centers

    def getPupilsCenterInUnbinnedPixels(self):
        return self._pupilsCenter

    def _computeFluxesForTilt(self):
        z2 = self._tipTiltCoefficients[0]
        z3 = self._tipTiltCoefficients[1]
        tot = 4.0
        tl = 0.25 * (z3 - z2 + 1) * tot
        tr = 0.25 * (z3 + z2 + 1) * tot
        bl = 0.25 * (-z3 - z2 + 1) * tot
        br = 0.25 * (-z3 + z2 + 1) * tot
        self._logger.debug(
            "tl,tr,bl,br %g,%g,%g,%g - z2,z3: %g,%g" %
            (tl, tr, bl, br, (tr + br - tl - bl) / tot,
             (tl + tr - bl - br) / tot))
        return tl, tr, bl, br

    def _pupilImagesFromWavefront(self, wavefront):
        Amat = np.array([[1., 0, 1, 0],
                        [0., 1, 0, 1],
                        [0., 0, 1, 1],
                        [1., 1, 0, 0]])

        (dyMap, dxMap) = np.gradient(wavefront / self._scaleInMeterPerPixel)
        dxMapClipped = dxMap.clip(min=-self._slopeSaturationInRad,
                                 max=self._slopeSaturationInRad)
        dyMapClipped = dyMap.clip(min=-self._slopeSaturationInRad,
                                 max=self._slopeSaturationInRad)
        dx = 2. / np.pi * np.arcsin(dxMapClipped / self._slopeSaturationInRad)
        dy = 2. / np.pi * np.arcsin(dyMapClipped / self._slopeSaturationInRad)
        (sz0, sz1) = dxMap.shape
        totalIntensity = 1.* sz0 * sz1
        B = totalIntensity / (sz0 * sz1) / 2 * np.array(
            [1 - dx, 1 + dx, 1 - dy, 1 + dy])

        res = np.linalg.lstsq(Amat, B.reshape((4, sz0 * sz1)), rcond=-1)
        pupils = res[0].reshape((4, sz0, sz1))

        if isinstance(wavefront, np.ma.MaskedArray):
            for i in range(4):
                pupils[i][wavefront.mask] = 0

        return pupils

    def _computeFrameFromWavefront(self):
        radius = self._pupilRadius
        cc = self._pupilsCenter
        maxNoiseInCount = self._noiseInCount

        pupils = self._pupilImagesFromWavefront(self._wavefront)
        pixels = np.zeros((self.SENSOR_H, self.SENSOR_W),
                          dtype=float)

        pixels[int(cc[0, 0] - radius): int(cc[0, 0] + radius),
               int(cc[0, 1] - radius): int(cc[0, 1] + radius)] = pupils[0]
        pixels[int(cc[1, 0] - radius): int(cc[1, 0] + radius),
               int(cc[1, 1] - radius): int(cc[1, 1] + radius)] = pupils[1]
        pixels[int(cc[2, 0] - radius): int(cc[2, 0] + radius),
               int(cc[2, 1] - radius): int(cc[2, 1] + radius)] = pupils[2]
        pixels[int(cc[3, 0] - radius): int(cc[3, 0] + radius),
               int(cc[3, 1] - radius): int(cc[3, 1] + radius)] = pupils[3]

        normalize = self._totalFluxPerMilliSecond * \
            self.exposureTime() / pixels.sum()
        pixels *= normalize

        if maxNoiseInCount > 0:
            pixels += np.random.normal(
                3 * maxNoiseInCount,
                maxNoiseInCount,
                size=(self.SENSOR_H, self.SENSOR_W))

        if self._binning != 1:
            pixels = rebin(pixels, self._binning) * self._binning ** 2
        pixels = pixels.clip(min=0, max=self.MAX_VALUE)

        self._counter += 1
        time.sleep(1. / self.getFrameRate())
        return pixels.astype(self.DTYPE)

    def setSlopeSaturationInRadians(self, slopeSaturationInRadians):
        self._slopeSaturationInRad = slopeSaturationInRadians

    def getSlopeSaturationInRadians(self):
        return self._slopeSaturationInRad

    def setScaleInMeterPerPixel(self, scaleInMeterPerPixel):
        self._scaleInMeterPerPixel = scaleInMeterPerPixel

    def getScaleInMeterPerPixel(self):
        return self._scaleInMeterPerPixel



#!/usr/bin/env python
import os
import sys
import subprocess
import shutil
import unittest
import logging
import numpy as np
from test.test_helper import TestHelper, Poller, MessageInFileProbe, \
    ExecutionProbe
from plico.utils.configuration import Configuration
from plico.rpc.zmq_remote_procedure_call import ZmqRemoteProcedureCall
from plico.utils.logger import Logger
from plico.rpc.sockets import Sockets
from plico.rpc.zmq_ports import ZmqPorts
from pysilico_server.utils.constants import Constants
from pysilico_server.utils.starter_script_creator import StarterScriptCreator
from pysilico_server.utils.process_startup_helper import ProcessStartUpHelper
from pysilico_server.process_monitor.runner import Runner as ProcessMonitorRunner
from pysilico_server.camera_controller.runner import Runner
from pysilico.client.camera_client import CameraClient
from pysilico.client.abstract_camera_client import SnapshotEntry
from pysilico_server.devices.simulated_auxiliary_camera import \
    SimulatedAuxiliaryCamera


@unittest.skipIf(sys.platform == "win32",
                 "Integration test doesn't run on Windows. Fix it!")
class IntegrationTest(unittest.TestCase):

    TEST_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                            "./tmp/")
    LOG_DIR = os.path.join(TEST_DIR, "log")
    CONF_FILE = 'test/integration/conffiles/pysilico_server.conf'
    CALIB_FOLDER = 'test/integration/calib'
    CONF_SECTION = Constants.PROCESS_MONITOR_CONFIG_SECTION
    SERVER_LOG_PATH = os.path.join(LOG_DIR, "%s.log" % CONF_SECTION)
    BIN_DIR = os.path.join(TEST_DIR, "apps", "bin")
    SOURCE_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                              "../..")

    def setUp(self):
        self._setUpBasicLogging()
        self.server = None
        self._wasSuccessful = False

        self._removeTestFolderIfItExists()
        self._makeTestDir()
        self.configuration = Configuration()
        self.configuration.load(self.CONF_FILE)
        self.rpc = ZmqRemoteProcedureCall()

        calibrationRootDir = self.configuration.calibrationRootDir()
        self._setUpCalibrationTempFolder(calibrationRootDir)

    def _setUpBasicLogging(self):
        logging.basicConfig(level=logging.DEBUG)
        self._logger = Logger.of('Integration Test')

    def _makeTestDir(self):
        os.makedirs(self.TEST_DIR)
        os.makedirs(self.LOG_DIR)
        os.makedirs(self.BIN_DIR)

    def _setUpCalibrationTempFolder(self, calibTempFolder):
        shutil.copytree(self.CALIB_FOLDER,
                        calibTempFolder)

    def _removeTestFolderIfItExists(self):
        if os.path.exists(self.TEST_DIR):
            shutil.rmtree(self.TEST_DIR)

    def tearDown(self):
        TestHelper.dumpFileToStdout(self.SERVER_LOG_PATH)

        if self.server is not None:
            TestHelper.terminateSubprocess(self.server)

        if self._wasSuccessful:
            self._removeTestFolderIfItExists()

    def _createStarterScripts(self):
        ssc = StarterScriptCreator()
        ssc.setInstallationBinDir(self.BIN_DIR)
        ssc.setPythonPath(self.SOURCE_DIR)
        ssc.setConfigFileDestination(self.CONF_FILE)
        numCameras = len(self.configuration.numberedSectionList(Constants.SERVER_CONFIG_SECTION_PREFIX))
        ssc.installExecutables(numCameras)

    def _startProcesses(self):
        psh = ProcessStartUpHelper()
        serverLog = open(os.path.join(self.LOG_DIR, "server.out"), "wb")
        self.server = subprocess.Popen(
            [psh.processProcessMonitorStartUpScriptPath(),
             self.CONF_FILE,
             self.CONF_SECTION])
           # stdout=serverLog, stderr=serverLog)
        Poller(5).check(MessageInFileProbe(
            ProcessMonitorRunner.RUNNING_MESSAGE, self.SERVER_LOG_PATH))

    def _testProcessesActuallyStarted(self):
        controllerLogFile = os.path.join(
            self.LOG_DIR,
            '%s%d.log' % (Constants.SERVER_CONFIG_SECTION_PREFIX, 1))
        Poller(5).check(MessageInFileProbe(
            Runner.RUNNING_MESSAGE, controllerLogFile))
        controller2LogFile = os.path.join(
            self.LOG_DIR,
            '%s%d.log' % (Constants.SERVER_CONFIG_SECTION_PREFIX, 2))
        Poller(5).check(MessageInFileProbe(
            Runner.RUNNING_MESSAGE, controller2LogFile))
        controller3LogFile = os.path.join(
            self.LOG_DIR,
            '%s%d.log' % (Constants.SERVER_CONFIG_SECTION_PREFIX, 3))
        Poller(5).check(MessageInFileProbe(
            Runner.RUNNING_MESSAGE, controller3LogFile))

    def _buildClients(self):
        ports1 = ZmqPorts.fromConfiguration(
            self.configuration,
            '%s%d' % (Constants.SERVER_CONFIG_SECTION_PREFIX, 1))
        self.client1 = CameraClient(
            self.rpc, Sockets(ports1, self.rpc))
        ports2 = ZmqPorts.fromConfiguration(
            self.configuration,
            '%s%d' % (Constants.SERVER_CONFIG_SECTION_PREFIX, 2))
        self.client2 = CameraClient(
            self.rpc, Sockets(ports2, self.rpc))
        ports3 = ZmqPorts.fromConfiguration(
            self.configuration,
            '%s%d' % (Constants.SERVER_CONFIG_SECTION_PREFIX, 3))
        self.client3 = CameraClient(
            self.rpc, Sockets(ports3, self.rpc))

    def _checkBackdoor(self):
        self.client1.execute(
            "import numpy as np; "
            "self._myarray= np.array([1, 2])")
        self.assertEqual(
            repr(np.array([1, 2])),
            self.client1.eval("self._myarray"))
        self.client1.execute("self._foobar= 42")
        self.assertEqual(
            "42",
            self.client1.eval("self._foobar"))

    def _testGetSnapshot(self):
        snapshot = self.client1.getSnapshot('aa')
        snKey = 'aa.%s' % SnapshotEntry.CAMERA_NAME
        self.assertEqual('Simulated Aux Camera', snapshot[snKey])

    def _testServerInfo(self):
        serverInfo = self.client1.serverInfo()
        self.assertEqual('AVT 1 server',
                         serverInfo.name)
        self.assertEqual('localhost', serverInfo.hostname)

    def _testCameraGetFrame(self):
        self._applyCameraBinning(self.client1, 1)
        cameraFrame = self.client1.getFutureFrames(1)
        frame = cameraFrame.toNumpyArray()
        counter = cameraFrame.counter()

        self.assertEqual(frame.shape,
                         (SimulatedAuxiliaryCamera.SENSOR_H,
                          SimulatedAuxiliaryCamera.SENSOR_W))

        counter2 = self.client1.getFutureFrames(1).counter()
        counter3 = self.client1.getFutureFrames(1).counter()
        self.assertTrue(counter2 > counter)
        self.assertTrue(counter3 > counter2)

    def _testCameraGetFutureFrames(self):
        self._applyCameraBinning(self.client1, 1)
        cameraFrame = self.client1.getFutureFrames(10)
        frame = cameraFrame.toNumpyArray()
        counter = cameraFrame.counter()

        self.assertEqual(
            frame.shape,
            (SimulatedAuxiliaryCamera.SENSOR_H,
             SimulatedAuxiliaryCamera.SENSOR_W, 10))
        self.assertEqual(float, frame.dtype)
        self.assertNotEqual(0,
                            np.std(frame[0, 0, :]))

        counter2 = self.client1.getFutureFrames(1).counter()
        self.assertTrue(counter2 > counter)

    def _testCameraModifyExposureTime(self):
        self.client1.setExposureTime(3.5)
        Poller(3).check(ExecutionProbe(
            lambda: self.assertEqual(3.5,
                                     self.client1.exposureTime())))
        self.client1.setExposureTime(0.75)
        Poller(3).check(ExecutionProbe(
            lambda: self.assertEqual(0.75,
                                     self.client1.exposureTime())))

    def _testCameraBinning(self):
        self._applyCameraBinning(self.client1, 1)
        shape1 = self.client1.getFutureFrames(1).toNumpyArray().shape
        Poller(3).check(ExecutionProbe(
            lambda: self.assertEqual(1,
                                     self.client1.getBinning())))
        self._applyCameraBinning(self.client1, 4)
        shape4 = self.client1.getFutureFrames(1).toNumpyArray().shape
        Poller(3).check(ExecutionProbe(
            lambda: self.assertEqual(4,
                                     self.client1.getBinning())))
        self.assertEqual(shape1[0] / 4, shape4[0])
        self.assertEqual(shape1[1] / 4, shape4[1])
        self._applyCameraBinning(self.client1, 1)

    def _applyCameraBinning(self, cameraClient, binning):
        cameraClient.setBinning(binning)
        # NOTNEEDED _trashLast= cameraClient.getFutureFrames(1)

    def _testFrameForDisplayIsResizedUnlessBinned(self):
        self._applyCameraBinning(self.client1, 1)
        disp = self.client1.getFrameForDisplay().toNumpyArray().shape
        full = self.client1.getFutureFrames(1).toNumpyArray().shape
        self.assertTrue(full[0] > disp[0])

        self._applyCameraBinning(self.client1, 8)
        disp = self.client1.getFrameForDisplay().toNumpyArray().shape
        full = self.client1.getFutureFrames(1).toNumpyArray().shape
        Poller(3).check(ExecutionProbe(
            lambda: self.assertEqual(
                full,
                self.client1.getFrameForDisplay().toNumpyArray().shape)))

    def testMain(self):
        self._buildClients()
        self._createStarterScripts()
        self._startProcesses()
        self._testProcessesActuallyStarted()
        self._testCameraGetFrame()
        self._testCameraGetFutureFrames()
        self._testCameraModifyExposureTime()
        self._testCameraBinning()
        self._testFrameForDisplayIsResizedUnlessBinned()
        self._testGetSnapshot()
        self._testServerInfo()
        self._checkBackdoor()
        self._wasSuccessful = True


if __name__ == "__main__":
    unittest.main()
